@@ -160,9 +160,9 @@ public:
 	float value() const
 		{ return m_value; }
 	float *value_ptr()
-		{ tick(); return &m_value; }
+		{ tick(1); return &m_value; }
 
-	virtual float tick(uint32_t /*nstep*/ = 1)
+	virtual float tick(uint32_t /*nstep*/)
 	{
 		if (m_port && ::fabsf(*m_port - m_vport) > 0.001f)
 			set_value(*m_port);
@@ -199,7 +199,7 @@ public:
 		synthv1_port::set_value(value);
 	}
 
-	float tick(uint32_t nstep = NSTEP)
+	float tick(uint32_t nstep)
 	{
 		if (m_nstep == 0)
 			return synthv1_port::tick(nstep);
@@ -973,11 +973,11 @@ synthv1_impl::~synthv1_impl (void)
 
 	delete [] m_voices;
 
-	// deallocate channels
-	setChannels(0);
-
 	// deallocate local buffers
 	alloc_sfxs(0);
+
+	// deallocate channels
+	setChannels(0);
 }
 
 
