@@ -597,6 +597,12 @@ void synthv1_jack::sessionEvent ( void *pvSessionArg )
 #endif	// CONFIG_JACK_SESSION
 
 
+void synthv1_jack::updatePreset ( bool /*bDirty*/ )
+{
+	// nothing to do here...
+}
+
+
 //-------------------------------------------------------------------------
 // synthv1_jack_application -- Singleton application instance.
 //
@@ -672,8 +678,12 @@ bool synthv1_jack_application::parse_args (void)
 		}
 		else
 		if (sArg == "-v" || sArg == "-V" || sArg == "--version") {
-			out << QObject::tr("Qt: %1\n").arg(qVersion());
-			out << QObject::tr(SYNTHV1_TITLE ": %1\n").arg(SYNTHV1_VERSION);
+			out << QObject::tr("Qt: %1\n")
+				.arg(qVersion());
+			out << QObject::tr("%1: %2  (%3)\n")
+				.arg(SYNTHV1_TITLE)
+				.arg(CONFIG_BUILD_VERSION)
+				.arg(CONFIG_BUILD_DATE);
 			return false;
 		}
 	}
