@@ -28,9 +28,25 @@
 #include "lv2/lv2plug.in/ns/ext/time/time.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
 
+#include "lv2/lv2plug.in/ns/ext/state/state.h"
+
 #include "lv2/lv2plug.in/ns/ext/options/options.h"
 #include "lv2/lv2plug.in/ns/ext/buf-size/buf-size.h"
 
+#ifndef CONFIG_LV2_ATOM_FORGE_OBJECT
+#define lv2_atom_forge_object(forge, frame, id, otype) \
+		lv2_atom_forge_blank(forge, frame, id, otype)
+#endif
+
+#ifndef CONFIG_LV2_ATOM_FORGE_KEY
+#define lv2_atom_forge_key(forge, key) \
+		lv2_atom_forge_property_head(forge, key, 0)
+#endif
+
+#ifndef LV2_STATE__StateChanged
+#define LV2_STATE__StateChanged LV2_STATE_PREFIX "StateChanged"
+#endif
+
 #include <stdlib.h>
 #include <math.h>
 
@@ -39,47 +55,61 @@
 // synthv1_lv2 - impl.
 //
 
+// atom-like message used internally with worker/schedule
+typedef struct {
+	LV2_Atom atom;
+} synthv1_lv2_worker_message;
+
+
 synthv1_lv2::synthv1_lv2 (
 	double sample_rate, const LV2_Feature *const *host_features )
 	: synthv1(2, float(sample_rate))
 {
 	::memset(&m_urids, 0, sizeof(m_urids));
 
-	m_atom_sequence = NULL;
+	m_urid_map = NULL;
+	m_atom_in  = NULL;
+	m_atom_out = NULL;
+	m_schedule = NULL;
+	m_ndelta   = 0;
 
 	const LV2_Options_Option *host_options = NULL;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		const LV2_Feature *host_feature = host_features[i];
 		if (::strcmp(host_feature->URI, LV2_URID_MAP_URI) == 0) {
-			LV2_URID_Map *urid_map
-				= (LV2_URID_Map *) host_feature->data;
-			if (urid_map) {
-				m_urids.atom_Blank = urid_map->map(
-					urid_map->handle, LV2_ATOM__Blank);
-				m_urids.atom_Object = urid_map->map(
-					urid_map->handle, LV2_ATOM__Object);
-				m_urids.atom_Float = urid_map->map(
-					urid_map->handle, LV2_ATOM__Float);
-				m_urids.atom_Int = urid_map->map(
-					urid_map->handle, LV2_ATOM__Int);
-				m_urids.time_Position = urid_map->map(
-					urid_map->handle, LV2_TIME__Position);
-				m_urids.time_beatsPerMinute = urid_map->map(
-					urid_map->handle, LV2_TIME__beatsPerMinute);
-				m_urids.midi_MidiEvent = urid_map->map(
-					urid_map->handle, LV2_MIDI__MidiEvent);
-				m_urids.bufsz_minBlockLength = urid_map->map(
-					urid_map->handle, LV2_BUF_SIZE__minBlockLength);
-				m_urids.bufsz_maxBlockLength = urid_map->map(
-					urid_map->handle, LV2_BUF_SIZE__maxBlockLength);
+			m_urid_map = (LV2_URID_Map *) host_feature->data;
+			if (m_urid_map) {
+				m_urids.atom_Blank = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Blank);
+				m_urids.atom_Object = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Object);
+				m_urids.atom_Float = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Float);
+				m_urids.atom_Int = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Int);
+				m_urids.time_Position = m_urid_map->map(
+					m_urid_map->handle, LV2_TIME__Position);
+				m_urids.time_beatsPerMinute = m_urid_map->map(
+					m_urid_map->handle, LV2_TIME__beatsPerMinute);
+				m_urids.midi_MidiEvent = m_urid_map->map(
+					m_urid_map->handle, LV2_MIDI__MidiEvent);
+				m_urids.bufsz_minBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__minBlockLength);
+				m_urids.bufsz_maxBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__maxBlockLength);
 			#ifdef LV2_BUF_SIZE__nominalBlockLength
-				m_urids.bufsz_nominalBlockLength = urid_map->map(
-					urid_map->handle, LV2_BUF_SIZE__nominalBlockLength);
+				m_urids.bufsz_nominalBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__nominalBlockLength);
 			#endif
+				m_urids.state_StateChanged = m_urid_map->map(
+					m_urid_map->handle, LV2_STATE__StateChanged);
 			}
 		}
 		else
+		if (::strcmp(host_feature->URI, LV2_WORKER__schedule) == 0)
+			m_schedule = (LV2_Worker_Schedule *) host_feature->data;
+		else
 		if (::strcmp(host_feature->URI, LV2_OPTIONS__options) == 0)
 			host_options = (const LV2_Options_Option *) host_feature->data;
 	}
@@ -108,6 +138,8 @@ synthv1_lv2::synthv1_lv2 (
 
 	synthv1::setBufferSize(buffer_size);
 
+	lv2_atom_forge_init(&m_forge, m_urid_map);
+
 	const uint16_t nchannels = synthv1::channels();
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
@@ -130,7 +162,10 @@ void synthv1_lv2::connect_port ( uint32_t port, void *data )
 {
 	switch(PortIndex(port)) {
 	case MidiIn:
-		m_atom_sequence = (LV2_Atom_Sequence *) data;
+		m_atom_in = (LV2_Atom_Sequence *) data;
+		break;
+	case Notify:
+		m_atom_out = (LV2_Atom_Sequence *) data;
 		break;
 	case AudioInL:
 		m_ins[0] = (float *) data;
@@ -160,10 +195,16 @@ void synthv1_lv2::run ( uint32_t nframes )
 		outs[k] = m_outs[k];
 	}
 
+	if (m_atom_out) {
+		const uint32_t capacity = m_atom_out->atom.size;
+		lv2_atom_forge_set_buffer(&m_forge, (uint8_t *) m_atom_out, capacity);
+		lv2_atom_forge_sequence_head(&m_forge, &m_notify_frame, 0);
+	}
+
 	uint32_t ndelta = 0;
 
-	if (m_atom_sequence) {
-		LV2_ATOM_SEQUENCE_FOREACH(m_atom_sequence, event) {
+	if (m_atom_in) {
+		LV2_ATOM_SEQUENCE_FOREACH(m_atom_in, event) {
 			if (event == NULL)
 				continue;
 			if (event->body.type == m_urids.midi_MidiEvent) {
@@ -198,7 +239,9 @@ void synthv1_lv2::run ( uint32_t nframes )
 				}
 			}
 		}
-	//	m_atom_sequence = NULL;
+		// remember last time for worker response
+		m_ndelta = ndelta;
+	//	m_atom_in = NULL;
 	}
 
 	if (nframes > ndelta)
@@ -256,6 +299,61 @@ void synthv1_lv2::select_program ( uint32_t bank, uint32_t program )
 #endif	// CONFIG_LV2_PROGRAMS
 
 
+uint32_t synthv1_lv2::urid_map ( const char *uri ) const
+{
+	return (m_urid_map ? m_urid_map->map(m_urid_map->handle, uri) : 0);
+}
+
+
+//-------------------------------------------------------------------------
+// synthv1_lv2 - LV2 State (bogus) interface.
+//
+
+void synthv1_lv2::updatePreset ( bool /*bDirty*/ )
+{
+	if (m_schedule /*&& bDirty*/) {
+		synthv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.state_StateChanged;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+bool synthv1_lv2::worker_work ( const void *data, uint32_t /*size*/ )
+{
+	const synthv1_lv2_worker_message *mesg
+		= (const synthv1_lv2_worker_message *) data;
+
+	return (mesg->atom.type == m_urids.state_StateChanged);
+}
+
+
+bool synthv1_lv2::worker_response ( const void *data, uint32_t /*size*/ )
+{
+	const synthv1_lv2_worker_message *mesg
+		= (const synthv1_lv2_worker_message *) data;
+
+	if (mesg->atom.type == m_urids.state_StateChanged)
+		return state_changed();
+	else
+		return false;
+}
+
+
+bool synthv1_lv2::state_changed (void)
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame frame;
+	lv2_atom_forge_object(&m_forge, &frame, 0, m_urids.state_StateChanged);
+	lv2_atom_forge_pop(&m_forge, &frame);
+
+	return true;
+}
+
+
 //-------------------------------------------------------------------------
 // synthv1_lv2 - LV2 desc.
 //
@@ -337,6 +435,40 @@ static const LV2_Programs_Interface synthv1_lv2_programs_interface =
 
 #endif	// CONFIG_LV2_PROGRAMS
 
+
+static LV2_Worker_Status synthv1_lv2_worker_work (
+	LV2_Handle instance, LV2_Worker_Respond_Function respond,
+	LV2_Worker_Respond_Handle handle, uint32_t size, const void *data )
+{
+	synthv1_lv2 *pSynth = static_cast<synthv1_lv2 *> (instance);
+	if (pSynth && pSynth->worker_work(data, size)) {
+		respond(handle, size, data);
+		return LV2_WORKER_SUCCESS;
+	}
+
+	return LV2_WORKER_ERR_UNKNOWN;
+}
+
+
+static LV2_Worker_Status synthv1_lv2_worker_response (
+	LV2_Handle instance, uint32_t size, const void *data )
+{
+	synthv1_lv2 *pSynth = static_cast<synthv1_lv2 *> (instance);
+	if (pSynth && pSynth->worker_response(data, size))
+		return LV2_WORKER_SUCCESS;
+	else
+		return LV2_WORKER_SUCCESS;
+}
+
+
+static const LV2_Worker_Interface synthv1_lv2_worker_interface =
+{
+	synthv1_lv2_worker_work,
+	synthv1_lv2_worker_response,
+	NULL
+};
+
+
 static const void *synthv1_lv2_extension_data ( const char *uri )
 {
 #ifdef CONFIG_LV2_PROGRAMS
@@ -344,6 +476,9 @@ static const void *synthv1_lv2_extension_data ( const char *uri )
 		return (void *) &synthv1_lv2_programs_interface;
 	else
 #endif
+	if (::strcmp(uri, LV2_WORKER__interface) == 0)
+		return &synthv1_lv2_worker_interface;
+
     return NULL;
 }
 
