@@ -1,7 +1,7 @@
 // synthv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -90,9 +90,6 @@ protected slots:
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
-	// Delay BPM change.
-	void bpmSyncChanged();
-
 	// Schedule notification updater.
 	void updateSchedNotify(int stype, int sid);
 
